--[[

Game:	Farming Simulator 19
Title:	UnitConvert Lite (Dialogs)
Author:	ThundR

--]]

-- [Declarations] ***********************************************************************************************************************************

UnitExtension.SiloDialog = {}

-- [Initialization] *********************************************************************************************************************************

function UnitExtension:initializeDialogs()
-- Inititializes Dialogs

	-- SiloDialog overrides
	HMUtils.setFunction(self, "SiloDialog", "setFillLevels")
	HMUtils.setFunction(self, "SiloDialog", "onClickFillTypes")
end

-- [SiloDialog Overrides] ***************************************************************************************************************************

function UnitExtension.SiloDialog:setFillLevels(...)
	local savedFunc = HMUtils.getSavedFunction(g_ucLite, "SiloDialog.setFillLevels")

	if not self.ucInitialized then
		self.fillTypesElement.onClickCallback = self.onClickFillTypes
		self.ucInitialized = true
	end

	local ret = HMUtils.pack(savedFunc(self, ...))
	return HMUtils.unpack(ret)
end

function UnitExtension.SiloDialog:onClickFillTypes(...)
	local savedFunc = HMUtils.getSavedFunction(g_ucLite, "SiloDialog.onClickFillTypes")
	local args = HMUtils.pack(...)

	local state = HMUtils.getRealArg(args, 1)

	local fillTypeIndex = self.fillTypeMapping[state]
	local fillTypeUnit, conversionFactor = g_ucLite:getFillTypeUnit(fillTypeIndex, "PRICE")
	local fillType = g_ucLite:getFillType(fillTypeIndex)
	local capacity = tonumber(self.capacity)

	if capacity and capacity < math.huge then
		local dialogTitle = self.dialogTitleElement.text
		if HMUtils.isString(dialogTitle) then
			local stationName, capacityIdx = string.gsub(dialogTitle, " ?%(%d.*$", "")
			if capacityIdx > 0 then
				capacity = capacity * conversionFactor
				capacity = HMUtils.numToStr(capacity, fillTypeUnit.precision)
				dialogTitle = string.format("%s (%s %s)", stationName, capacity, fillTypeUnit.titleShort)
				self.dialogTitleElement:setText(dialogTitle)
			end
		end
	end

	local fillLevel = tonumber(self.fillLevels[fillType.index])
	if fillLevel and fillLevel < math.huge then
		fillLevel = fillLevel * conversionFactor
		fillLevel = HMUtils.numToStr(fillLevel, fillTypeUnit.precision)
		fillTypeText = string.format("%s: %s %s", fillType.title, fillLevel, fillTypeUnit.titleShort)
		self.fillTypesElement.texts[state] = fillTypeText
		self.siloText:setText(fillTypeText)
	end

	local ret = HMUtils.pack(savedFunc(self, ...))
	return HMUtils.unpack(ret)
end
