--[[

Game:	Farming Simulator 19
Title:	UnitConvert Lite (InGameMenu)
Author:	ThundR

--]]

-- [Declarations] ***********************************************************************************************************************************

UnitExtension.InGameMenuMapFrame = {}
UnitExtension.InGameMenuPricesFrame = {}
UnitExtension.InGameMenuStatisticsFrame = {}

-- [Initialization] *********************************************************************************************************************************

function UnitExtension:initializeInGameMenu()
-- Initializes InGameMenu

	-- InGameMenuMapFrame overrides
	HMUtils.setFunction(self, "InGameMenuMapFrame", "onFrameOpen")
	HMUtils.setFunction(self, "InGameMenuMapFrame", "generateFarmlandOverlay")

	-- InGameMenuPricesFrame overrides
	HMUtils.setFunction(self, "InGameMenuPricesFrame", "buildSiloRow")
	HMUtils.setFunction(self, "InGameMenuPricesFrame", "buildDataRow")
	HMUtils.setFunction(self, "InGameMenuPricesFrame", "updatePriceTable")

	-- InGameMenuStatisticsFrame overrides
	HMUtils.setFunction(self, "InGameMenuStatisticsFrame", "buildDataRow")
end

-- [InGameMenuMapFrame Overrides] *******************************************************************************************************************

function UnitExtension.InGameMenuMapFrame:onFrameOpen(...)
	local savedFunc = HMUtils.getSavedFunction(g_ucLite, "InGameMenuMapFrame.onFrameOpen")

	self.unitExtension = Utils.getNoNil(self.unitExtension, {})

	if not self.unitExtension.initialized then
		local farmlandBalanceParent = self.balanceText.parent
		local farmlandValueParent = self.farmlandValueText.parent
		local farmlandAreaElement = farmlandValueParent:clone(self.farmlandValueBox)
		local farmlandOwnerElement = farmlandValueParent:clone(self.farmlandValueBox)

		-- Owner element
		local parentX, parentY = HMUtils.unpack(farmlandBalanceParent.position)
		local parentWidth, parentHeight = HMUtils.unpack(farmlandBalanceParent.size)
		farmlandOwnerElement:updateScreenAlign("middleLeft")
		farmlandOwnerElement:updatePositionForOrigin("middleLeft")
		farmlandOwnerElement:setPosition(parentX, parentY + parentHeight)
		farmlandOwnerElement.elements[1]:setText(g_ucLite:getLocalText("ui_owner")..":")
		self.farmlandOwnerElement = farmlandOwnerElement
		self.farmlandOwnerText = farmlandOwnerElement.elements[2]

		-- Area element
		local parentX, parentY = HMUtils.unpack(farmlandValueParent.position)
		local parentWidth, parentHeight = HMUtils.unpack(farmlandValueParent.size)
		farmlandAreaElement:setPosition(parentX, parentY + parentHeight)
		farmlandAreaElement.elements[1]:setText(HMUtils.getText("ui_fieldArea")..":")
		self.farmlandAreaElement = farmlandAreaElement
		self.farmlandAreaText = farmlandAreaElement.elements[2]

		self.unitExtension.initialized = true
	end

	local ret = HMUtils.pack(savedFunc(self, ...))
	return HMUtils.unpack(ret)
end

function UnitExtension.InGameMenuMapFrame:generateFarmlandOverlay(...)
	local savedFunc = HMUtils.getSavedFunction(g_ucLite, "InGameMenuMapFrame.generateFarmlandOverlay")
	local ret = HMUtils.pack(savedFunc(self, ...))

	local ownerText = ""
	local areaText = "0"

	local selectedFarmland = self.selectedFarmland
	if selectedFarmland then
		local area = tonumber(selectedFarmland.areaInHa)
		local npcIndex = g_npcManager:getNPCByIndex(tonumber(selectedFarmland.npcIndex))

		if selectedFarmland.isOwned then
			ownerText = HMUtils.getText("ui_playerCharacter")
		elseif npcIndex then
			ownerText = npcIndex.title
		end

		if area then
			area = area * g_i18n:getArea(g_ucLite.areaScale)
			areaText = HMUtils.numToStr(area, 2, "ROUND")
		end
	end

	self.farmlandOwnerText:setText(ownerText)
	self.farmlandAreaText:setText(string.format("%s %s", areaText, g_i18n:getAreaUnit()))

	return HMUtils.unpack(ret)
end

-- [InGameMenuPricesFrame Overrides] ****************************************************************************************************************

function UnitExtension.InGameMenuPricesFrame:buildSiloRow(...)
	local savedFunc = HMUtils.getSavedFunction(g_ucLite, "InGameMenuPricesFrame.buildSiloRow")
	local ret = HMUtils.pack(savedFunc(self, ...))

	if ret.n > 0 then
		local data = ret[1]
		local startFillType = self.startFillType

		for i = 1, self.MAX_NUM_FILLTYPES do
			local fillTypeIndex = startFillType + (i-1)
			local priceFillType = self.priceFillTypes[fillTypeIndex]
			if priceFillType then
				local fillTypeUnit, conversionFactor = g_ucLite:getFillTypeUnit(priceFillType.index, "PRICE")
				local priceCell = data.columnCells["price"..tostring(i)]
				local capacityCell = data.columnCells["capacity"..tostring(i)]

				if priceCell then
					local fillLevel = Utils.getNoNil(HMUtils.strToNum(priceCell.text), 0)
					if fillLevel > 0 and fillLevel < math.huge then
						fillLevel = fillLevel * conversionFactor
						fillLevel = HMUtils.numToStr(fillLevel)
						priceCell.text = string.format("%s %s", fillLevel, fillTypeUnit.titleShort)
					else
						priceCell.text = "-"
					end
				end

				if capacityCell then
					local capacity = Utils.getNoNil(HMUtils.strToNum(capacityCell.text), 0)
					if capacity <= 0 then
						priceCell.text = "-"
					end

					if capacity >= 0 and capacity < math.huge then
						capacity = capacity * conversionFactor
						capacity = HMUtils.numToStr(capacity)
						capacityCell.text = string.format("%s %s", capacity, fillTypeUnit.titleShort)
					end
				end
			end
		end
		ret[1] = data
	end

	return HMUtils.unpack(ret)
end

function UnitExtension.InGameMenuPricesFrame:buildDataRow(...)
	local savedFunc = HMUtils.getSavedFunction(g_ucLite, "InGameMenuPricesFrame.buildDataRow")
	local ret = HMUtils.pack(savedFunc(self, ...))

	if ret.n > 0 then
		local data = ret[1]
		local startFillType = self.startFillType

		for i = 1, self.MAX_NUM_FILLTYPES do
			local fillTypeIndex = startFillType + (i-1)
			local priceFillType = self.priceFillTypes[fillTypeIndex]
			if priceFillType then
				local fillTypeUnit, conversionFactor = g_ucLite:getFillTypeUnit(priceFillType.index, "PRICE")
				local priceCell = data.columnCells["price"..tostring(i)]
				if priceCell then
					local price = tonumber(priceCell.value)
					if price and price > 0 then
						price = price / conversionFactor
						price = HMUtils.numToStr(price, 2, "ROUND", false, true)
						priceCell.text = string.format("%s %s", price, g_i18n:getCurrencySymbol(true))
					else
						priceCell.text = "-"
					end
				end
			end
		end
		ret[1] = data
	end
	return HMUtils.unpack(ret)
end

function UnitExtension.InGameMenuPricesFrame:updatePriceTable(...)
	local savedFunc = HMUtils.getSavedFunction(g_ucLite, "InGameMenuPricesFrame.updatePriceTable")

	local pageTitleElement = self.mainBox.parent.elements[1].elements[2]

	-- Initialize prices menu
	if not self.ucInitialized then
		-- Set page title
		pageTitleElement:setText(g_ucLite:getLocalText("ui_ingameMenuPrices"))

		-- Create fill type name elements
		self.fillTypeNameHeaders = {}
		local tableHeaderBox = self.tableHeaderBox
		for i, element in ipairs(tableHeaderBox.elements) do
			if i > 1 then
				local fillTypeNameElement = TextElement:new(self)
				element:addElement(fillTypeNameElement)
				fillTypeNameElement:applyProfile("ingameMenuPriceRowSiloCapacityValue")
				table.insert(self.fillTypeNameHeaders, fillTypeNameElement)
			end
		end
		self.ucInitialized = true
	end

	local ret = HMUtils.pack(savedFunc(self, ...))

	local fillTypeNameHeaders = self.fillTypeNameHeaders
	local startFillType = self.startFillType

	for columnIdx = 1, self.MAX_NUM_FILLTYPES do
		local fillTypeIndex = startFillType + (columnIdx-1)
		local fillType = self.priceFillTypes[fillTypeIndex]

		local fillTypeNameHeader = fillTypeNameHeaders[columnIdx]
		if fillTypeNameHeader then
			if fillType then
				fillTypeNameHeader:setText(HMUtils.properCase(fillType.title))
				fillTypeNameHeader:setVisible(true)
			else
				fillTypeNameHeader:setText("")
				fillTypeNameHeader:setVisible(false)
			end
		end
	end
	return HMUtils.unpack(ret)
end

-- [InGameMenuStatisticsFrame Overrides] ************************************************************************************************************

function UnitExtension.InGameMenuStatisticsFrame:buildDataRow(...)
	local savedFunc = HMUtils.getSavedFunction(g_ucLite, "InGameMenuStatisticsFrame.buildDataRow")
	local ret = HMUtils.pack(savedFunc(self, ...))

	if ret.n > 0 then
		local data = ret[1]
		if HMUtils.isTable(data) then
			if data.columnCells and data.id then
				local id = data.id
				local fillTypeIndex = nil
				if string.find(id, HMUtils.getText("unit_liter")) then
					if string.find(id, HMUtils.getText("statistic_fuelUsage")) then
						fillTypeIndex = FillType.DIESEL
					elseif string.find(id, HMUtils.getText("statistic_seedUsage")) then
						fillTypeIndex = FillType.SEEDS
					elseif string.find(id, HMUtils.getText("statistic_sprayUsage")) then
						fillTypeIndex = FillType.FERTILIZER -- TODO: Get an average value here
					end

					if fillTypeIndex then
						local fillTypeUnit, conversionFactor = g_ucLite:getFillTypeUnit(fillTypeIndex, "PRICE")
						local statTable = HMUtils.getTable(data, "columnCells.stat")

						if statTable then
							local statText = tostring(statTable.text)
							if statText then
								statText = string.gsub(statText, HMUtils.getText("unit_liter"), fillTypeUnit.title)
								statTable.text = statText
							end
						end

						local sessionValueTable = HMUtils.getTable(data, "columnCells.sessionValue")
						if sessionValueTable then
							local sessionValue = tonumber(sessionValueTable.text)
							if sessionValue and sessionValue > 0 then
								sessionValue = sessionValue * conversionFactor
								sessionValueTable.text = HMUtils.numToStr(sessionValue, 2, "FLOOR")
							end
						end

						local totalValueTable = HMUtils.getTable(data, "columnCells.totalValue")
						if totalValueTable then
							local totalValue = tonumber(totalValueTable.text)
							if totalValue and totalValue > 0 then
								totalValue = totalValue * conversionFactor
								totalValueTable.text = HMUtils.numToStr(totalValue, 2, "FLOOR")
							end
						end
					end
				end
			end
		end
		ret[1] = data
	end
	return HMUtils.unpack(ret)
end