--[[

Game:	Farming Simulator 19
Title:	UnitConvert Lite (SpeedMeterDisplay)
Author:	ThundR

--]]

-- [Declarations] ***********************************************************************************************************************************

UnitExtension.SpeedMeterDisplay = {}

UnitExtension.SpeedMeterDisplay.POSITION = {}
UnitExtension.SpeedMeterDisplay.SIZE = {}
UnitExtension.SpeedMeterDisplay.UV = {}

UnitExtension.SpeedMeterDisplay.POSITION.WORKED_AREA = {unpack(SpeedMeterDisplay.POSITION.OPERATING_TIME)}
UnitExtension.SpeedMeterDisplay.SIZE.WORKED_AREA = {19.8, 28.2}
UnitExtension.SpeedMeterDisplay.UV.WORKED_AREA = {246, 297, 36, 31}

-- [Initialization] *********************************************************************************************************************************

function UnitExtension:initializeSpeedMeterDisplay()
-- Initializes SpeedMeterDisplay

	-- SpeedMeterDisplay additional functions
	HMUtils.setFunction(self, "SpeedMeterDisplay", "createWorkedAreaElement", false)
	HMUtils.setFunction(self, "SpeedMeterDisplay", "updateWorkedArea", false)

	-- SpeedMeterDisplay overrides
	HMUtils.setFunction(self, "SpeedMeterDisplay", "update")
	HMUtils.setFunction(self, "SpeedMeterDisplay", "createComponents")
	HMUtils.setFunction(self, "SpeedMeterDisplay", "updateDamageGauge")
	HMUtils.setFunction(self, "SpeedMeterDisplay", "updateOperatingTime")
	HMUtils.setFunction(self, "SpeedMeterDisplay", "drawOperatingTimeText")
end

-- [SpeedMeterDisplay Additional Functions] *********************************************************************************************************

function UnitExtension.SpeedMeterDisplay:createWorkedAreaElement(hudAtlasPath, baseX, baseY)
-- Creates a new worked area element based on the operating hours element
	-- [hudAtlasPath]:	Path to overlay image file
	-- [baseX]:			Base X position of element
	-- [baseY]:			Base Y position of element

	if HMUtils.argIsInvalid(HMUtils.isString(hudAtlasPath), "hudAtlasPath")
	or HMUtils.argIsInvalid(HMUtils.isNumber(baseX), "baseX")
	or HMUtils.argIsInvalid(HMUtils.isNumber(baseY), "baseY")
	then
		return
	end

	local workedAreaWidth, workedAreaHeight = getNormalizedScreenValues(unpack(UnitExtension.SpeedMeterDisplay.SIZE.WORKED_AREA))
	local workedAreaOffsetX, workedAreaOffsetY = getNormalizedScreenValues(unpack(UnitExtension.SpeedMeterDisplay.POSITION.WORKED_AREA))

	local workedAreaOverlay = Overlay:new(hudAtlasPath, baseX + workedAreaOffsetX, baseY + workedAreaOffsetY, workedAreaWidth, workedAreaHeight)
	workedAreaOverlay:setUVs(getNormalizedUVs(UnitExtension.SpeedMeterDisplay.UV.WORKED_AREA))

	local element = HUDElement:new(workedAreaOverlay)
   	self:addChild(element)
   	return element
end

function UnitExtension.SpeedMeterDisplay:updateWorkedArea()
-- Updates worked area element

	local workedArea = self.vehicle.unitExtension.totalWorkedArea
	local pixelsPerSqm = g_currentMission:getFruitPixelsToSqm()

	workedArea = MathUtil.areaToHa(workedArea, pixelsPerSqm) * g_i18n:getArea(g_ucLite.areaScale)
	workedArea = HMUtils.numToStr(workedArea, 2, "FLOOR")
	self.workedAreaText = string.format("%s %s", workedArea, g_i18n:getAreaUnit())

   	local textWidth = getTextWidth(self.workedAreaTextSize, self.workedAreaText)
   	local workedAreaWidth = self.workedAreaElement:getWidth() + self.workedAreaTextOffsetX + textWidth
   	local posX, _ = self:getPosition()
   	local _, posY = self.workedAreaElement:getPosition()

   	posX = posX + ((self:getWidth() - workedAreaWidth) * 0.5)

	self.workedAreaTextDrawPositionX = posX + self.workedAreaElement:getWidth() + self.workedAreaTextOffsetX
   	self.workedAreaTextDrawPositionY = posY + self.workedAreaTextOffsetY
   	self.workedAreaElement:setPosition(posX, nil)
end

-- [SpeedMeterDisplay Overrides] ********************************************************************************************************************

function UnitExtension.SpeedMeterDisplay:update(...)
	local savedFunc = HMUtils.getSavedFunction(g_ucLite, "SpeedMeterDisplay.update")
	local ret = HMUtils.pack(savedFunc(self, ...))
	return unpack(ret)
end

function UnitExtension.SpeedMeterDisplay:createComponents(...)
	local savedFunc = HMUtils.getSavedFunction(g_ucLite, "SpeedMeterDisplay.createComponents")
	local ret = HMUtils.pack(savedFunc(self, ...))

	local uiAtlasPath = "dataS2/menu/hud/ui_elements_1080p.png"
	local baseX, baseY = self:getBasePosition()

	self.workedAreaElement = self:createWorkedAreaElement(uiAtlasPath, baseX, baseY)
	self.workedAreaElement:setVisible(false)

	self.workedAreaText = ""
	self.workedAreaTextSize = self.operatingTimeTextSize
	self.workedAreaTextOffsetX = self.operatingTimeTextOffsetX
	self.workedAreaTextOffsetY = self.operatingTimeTextOffsetY
	self.workedAreaTextDrawPositionX = 0
	self.workedAreaTextDrawPositionY = 0

	return HMUtils.unpack(ret)
end

function UnitExtension.SpeedMeterDisplay:updateDamageGauge(...)
	local savedFunc = HMUtils.getSavedFunction(g_ucLite, "SpeedMeterDisplay.updateDamageGauge")

	local oldVehicle = self.vehicle
	if oldVehicle then
		self.vehicle = Utils.getNoNil(oldVehicle:getSelectedVehicle(), self.vehicle)
	end

	local needDamageGauge = self.vehicle.getWearTotalAmount ~= nil and self.vehicle:getWearTotalAmount() ~= nil
	self.damageGaugeActive = needDamageGauge
	self:animateDamageGaugeToggle(needDamageGauge)

	local ret = HMUtils.pack(savedFunc(self, ...))

	if oldVehicle then
		self.vehicle = oldVehicle
	end

	return HMUtils.unpack(ret)
end

function UnitExtension.SpeedMeterDisplay:updateOperatingTime(...)
	local savedFunc = HMUtils.getSavedFunction(g_ucLite, "SpeedMeterDisplay.updateOperatingTime")

	local oldVehicle = self.vehicle
	if oldVehicle then
		self.vehicle = Utils.getNoNil(oldVehicle:getSelectedVehicle(), self.vehicle)
	end

	local hasOperatingTime = self.vehicle.operatingTime ~= nil
	local hasWorkedArea = self.vehicle.unitExtension.totalWorkedArea ~= nil

	local ret = HMUtils.pack(savedFunc(self, ...))

	-- Worked Area (overrides operating time when toggled)
	if hasWorkedArea and self.vehicle.unitExtension.showWorkedArea == true then
		self.operatingTimeElement:setVisible(false)
		self.workedAreaElement:setVisible(true)
		self:updateWorkedArea(...)
	else
		self.operatingTimeElement:setVisible(hasOperatingTime)
		self.workedAreaElement:setVisible(false)
	end

	if oldVehicle then
		self.vehicle = oldVehicle
	end

	return HMUtils.unpack(ret)
end

function UnitExtension.SpeedMeterDisplay:drawOperatingTimeText(...)
	local savedFunc = HMUtils.getSavedFunction(g_ucLite, "SpeedMeterDisplay.drawOperatingTimeText")
	local ret = HMUtils.pack(savedFunc(self, ...))

	if self.workedAreaElement:getVisible() then
        setTextAlignment(RenderText.ALIGN_LEFT)
        setTextBold(false)
        setTextColor(1, 1, 1, 1)
        renderText(self.workedAreaTextDrawPositionX, self.workedAreaTextDrawPositionY, self.workedAreaTextSize, self.workedAreaText)
	end

	return unpack(ret)
end
